@@ -42,19 +42,6 @@ def set_is_exiting():
     exiting = True
 
 
-def ask_master_to_restart():
-    data = [{
-        "when": BaseTask.get_current_time_in_users_time_zone(),
-        "data": {
-            "task_id": BaseTask.task_id
-        }
-    }]
-
-    r = requests.post(f"{BaseTask.MASTER_API}/tasks/restart/",
-                      json=data)
-    r.raise_for_status()
-    # Wait For Process to be restarted by Master
-    sleep_forever()
 
 
 def execute_after_n_seconds(f, s):
@@ -99,35 +86,49 @@ class RerunException(Exception):
 NULL = "NULL"
 
 
-def has_user_id():
-    return "user_id" in BaseTask.user
-
-
 class BaseTask():
-    data = json.loads(os.environ.get('data')) if os.environ.get('data') is not None else {} 
-    user =  json.loads(os.environ.get('user')) if os.environ.get('user') is not None else {}
-    task_type = os.environ.get('task_type')
-    MODULE_NAME = os.environ.get('MODULE_NAME', '')
+    sys_exit_on_crash = True
+    def __init__(self, user = None):
+        self.data = json.loads(os.environ.get('data')) if os.environ.get('data') is not None else {} 
+        self.user = user if user is not None else json.loads(os.environ.get('user')) if os.environ.get('user') is not None else {}
+        
+        self.task_type = os.environ.get('task_type')
+        self.MODULE_NAME = os.environ.get('MODULE_NAME', '')
 
-    task_id = os.environ.get('task_id')
-    SKIP_IP_CHECK = get_boolean_variable('SKIP_IP_CHECK') if os.environ.get('SKIP_IP_CHECK') is not None else True
-     
-    is_debug = not IS_DOCKER
+        self.task_id = os.environ.get('task_id')
+        self.SKIP_IP_CHECK = get_boolean_variable('SKIP_IP_CHECK') if os.environ.get('SKIP_IP_CHECK') is not None else True
+        
+        self.is_debug = not IS_DOCKER
+
+        self.MASTER_API = MASTER_API_URL
+
+
+    def ask_master_to_restart(self):
+        data = [{
+            "when": self.get_current_time_in_users_time_zone(),
+            "data": {
+                "task_id": self.task_id
+            }
+        }]
+
+        r = requests.post(f"{self.MASTER_API}/tasks/restart/",
+                        json=data)
+        r.raise_for_status()
+        # Wait For Process to be restarted by Master
+        sleep_forever()
 
-    MASTER_API = MASTER_API_URL
 
     def run(self, driver: MyChrome, data, user):
         pass
-
     
     def sign_up_success(self):
-        r = requests.post(f"{BaseTask.MASTER_API}/tasks/sign-up-success/",
-                          json={'task_id': BaseTask.task_id})
+        r = requests.post(f"{self.MASTER_API}/tasks/sign-up-success/",
+                          json={'task_id': self.task_id})
         r.raise_for_status()
         return True
 
     def save_failure_screenshot(student):
-        save_screenshot(student.driver, BaseTask.user["user_id"],
+        save_screenshot(student.driver, self.user["user_id"],
                         SAVE_SCREENSHOT_SUCCESS_FOLDER, 'success')
 
     def assert_bot_has_not_been_caught(self):
@@ -136,9 +137,9 @@ class BaseTask():
         if is_caught:
             raise BotCaughtException(message)
 
-    def register_ip_for_task(ip):
-        r = requests.post(f"{BaseTask.MASTER_API}/tasks/register-ip/",
-                          json={'ip': ip, 'task_id': BaseTask.task_id})
+    def register_ip_for_task(self, ip):
+        r = requests.post(f"{self.MASTER_API}/tasks/register-ip/",
+                          json={'ip': ip, 'task_id': self.task_id})
 
         if r.status_code == 400:
             print('Ip already registered')
@@ -154,36 +155,36 @@ class BaseTask():
     register_ip = True
 
     def save_success_screenshot(self):
-            save_screenshot(self.driver , BaseTask.user["user_id"],
+            save_screenshot(self.driver , self.user["user_id"],
                                 SAVE_SCREENSHOT_SUCCESS_FOLDER, 'success')
 
     def begin_task(self):
             # Always use undetected in docker
         if IS_DOCKER:
             self.DRIVER_TYPE = UNDETECTED_DRIVER
-        if BaseTask.SKIP_IP_CHECK:
+        if self.SKIP_IP_CHECK:
             print(f'Skipping IP Check')
         else:
             ip = wait_till_ip_change()
             
             if self.register_ip:
-                is_success = BaseTask.register_ip_for_task(ip)
+                is_success = self.register_ip_for_task(ip)
                 
                 if not is_success:
-                    ask_master_to_restart()
+                    self.ask_master_to_restart()
                 # Skip IP check next time
-            BaseTask.SKIP_IP_CHECK = True
+            self.SKIP_IP_CHECK = True
 
-        if not has_user_id():
-            BaseTask.user["user_id"] = NULL
+        if not self.has_user_id():
+            self.user["user_id"] = NULL
 
-        is_sign_up_task = 'sign_up' in BaseTask.MODULE_NAME.lower()
+        is_sign_up_task = 'sign_up' in self.MODULE_NAME.lower()
 
-        self.user = BaseTask.user
-        self.data = BaseTask.data
+        self.user = self.user
+        self.data = self.data
 
-        data = BaseTask.data
-        user = BaseTask.user
+        data = self.data
+        user = self.user
 
         def do_run():
             driver = self.create_driver(self.is_eager)
@@ -199,7 +200,7 @@ class BaseTask():
                 if is_sign_up_task:
                     self.sign_up_success()
 
-                BaseTask.close(driver)
+                self.close(driver)
                 print('Bot ran succesfully')
 
             except Exception as error:
@@ -227,22 +228,22 @@ class BaseTask():
                                     
                     save_html_logs(driver, user.get( "user_id", 'NULL'))
 
-                BaseTask.close(driver)
+                self.close(driver)
 
                 if isinstance(error, BotCaughtException):
                     print('Deletig Profile')
                     if user.get( "user_id") is not None:
-                        delete_profile_path(BaseTask.user["user_id"])
-                    ask_master_to_restart()
+                        delete_profile_path(self.user["user_id"])
+                    self.ask_master_to_restart()
                 elif isinstance(error, EmailAlreadyExists):
                     print('Deletig Profile')
                     if user.get( "user_id") is not None:
-                        delete_profile_path(BaseTask.user["user_id"])
-                    BaseTask.change_email()
-                    ask_master_to_restart()
+                        delete_profile_path(self.user["user_id"])
+                    self.change_email()
+                    self.ask_master_to_restart()
 
                 if isinstance(error, RetryException):
-                    ask_master_to_restart()
+                    self.ask_master_to_restart()
                 if isReRun:
                     if self.retries < self.max_retries:
                         self.retries+=1
@@ -253,15 +254,16 @@ class BaseTask():
                         do_run()
                     
                 else:
-                    exit_with_failed_status()
+                    if self.sys_exit_on_crash:
+                        exit_with_failed_status()
         do_run()
 
     retries = 0
     max_retries = math.inf  
     
-    def change_email():
-        r = requests.patch(f"{BaseTask.MASTER_API}/tasks/change-email/",
-                           json={'task_id': BaseTask.task_id})
+    def change_email(self):
+        r = requests.patch(f"{self.MASTER_API}/tasks/change-email/",
+                           json={'task_id': self.task_id})
         r.raise_for_status()
         return True
 
@@ -269,7 +271,7 @@ class BaseTask():
 
 
     def has_user_id(self):
-        return  BaseTask.user.get("user_id") != NULL and BaseTask.user.get("user_id") is not None
+        return  self.user.get("user_id") != NULL and self.user.get("user_id") is not None
     FORCE_VERSION = None
 
     def create_driver(self, is_eager):
@@ -280,14 +282,14 @@ class BaseTask():
                 driver_type = self.DRIVER_TYPE
                 return create_driver_by_driver_type(
                     driver_type,
-                    BaseTask.user.get("user_id") if self.has_user_id() else None,
-                    BaseTask.user.get("window_size", RANDOM),
-                    BaseTask.user.get("user_agent", RANDOM),
+                    self.user.get("user_id") if self.has_user_id() else None,
+                    self.user.get("window_size", RANDOM),
+                    self.user.get("user_agent", RANDOM),
                     is_eager,
                     self.FORCE_VERSION)
 
             driver = retry_if_is_error(
-                run, NETWORK_ERRORS + [(WebDriverException, lambda: delete_corrupted_files(BaseTask.user["user_id"]) if self.has_user_id() else None )], RETRY_ATTEMPTS_HIGH)
+                run, NETWORK_ERRORS + [(WebDriverException, lambda: delete_corrupted_files(self.user["user_id"]) if self.has_user_id() else None )], RETRY_ATTEMPTS_HIGH)
 
             def call_quit(signalNumber, frame):
                 if is_exiting():
@@ -298,7 +300,7 @@ class BaseTask():
 
                 print('Force Exit Detected. Exiting Gracefully')
                 try:
-                    BaseTask.close(driver)
+                    self.close(driver)
                 except Exception:
                     print('Someting went wrong closing Driver')
                 finally:
@@ -311,13 +313,13 @@ class BaseTask():
 
         # Basically ask master to restart if failed to start after 10 minutes
         thread = multiprocessing.Process(
-            target=execute_after_n_seconds, args=(ask_master_to_restart, 600))
+            target=execute_after_n_seconds, args=(self.ask_master_to_restart, 600))
         thread.start()
         driver = do_create_driver()
         thread.terminate()
         return driver
 
-    def close(driver):
+    def close(self, driver):
         delete_cache(driver)
 
         print("Closing Driver")
@@ -328,7 +330,7 @@ class BaseTask():
 
     def get_current_time_in_users_time_zone():
         result = datetime_to_str(utc_to_timezoned_datetime(
-            get_time_in_utc(), country_code_to_time_zone(BaseTask.user["country_code"])))
+            get_time_in_utc(), country_code_to_time_zone(self.user["country_code"])))
 
         return result
 
