@@ -11,7 +11,7 @@ from time import sleep
 import requests
 from requests.exceptions import ReadTimeout
 import traceback
-
+from pydash import uniq_by
 import os
 from .config import IS_DOCKER
 
@@ -172,11 +172,15 @@ def write_temp_json(data, indent=4):
         json.dump(data, fp, indent=indent)
 
 
+
 def read_json(path):
     with open(path, 'r') as fp:
         users = json.load(fp)
         return users
 
+def read_temp_json():
+    path = './temp.json'
+    return read_json(path)
 
 def read_file(path):
     with open(path, 'r') as fp:
@@ -409,6 +413,9 @@ def write_csv(data):
     print(f'Writing {path}')
     df.to_csv(path, encoding='utf-8', index=False)
 
+def unique_by_id(objects):
+    return uniq_by(objects, 'id')
+
 if __name__ == '__main__':
     # print(get_utc_time())
     # print(datetime.now())
