@@ -48,6 +48,9 @@ def get_domains():
 
 
 class TempMail():
+    def get_domains():
+        return get_domains()
+
     def generate_email(username):
         # ['1secmail.com', '1secmail.net', '1secmail.org']
         domain = random.choice(get_domains())
@@ -72,7 +75,7 @@ class TempMail():
 
         requests.post(url, data=data)
 
-    def get_email_link_and_delete_mailbox(email):
+    def get_email_link(email):
         def run():
             login, domain = TempMail.extract(email)
             reqLink = f'{API}?action=getMessages&login={login}&domain={domain}'
@@ -88,7 +91,7 @@ class TempMail():
             req = requests.get(msgRead).json()
 
             html = req['htmlBody']
-
+            
             if html == '':
                 links = extract_links_from_text(req['textBody'])
             else:
@@ -101,11 +104,43 @@ class TempMail():
             return links[0]
 
         retry_if_is_error(
-            run, NETWORK_ERRORS + [AssertionError], RETRY_ATTEMPTS_HIGH, WAIT_DURATION_SHORT)
+            run, NETWORK_ERRORS + [AssertionError], RETRY_ATTEMPTS_HIGH, WAIT_DURATION_VERY_SHORT)
         link = run()
-        TempMail.deleteMailbox(email)
         return link
 
+    def get_body(email):
+        def run():
+            login, domain = TempMail.extract(email)
+            reqLink = f'{API}?action=getMessages&login={login}&domain={domain}'
+            req = requests.get(reqLink).json()
+
+            if len(req) == 0:
+                print('No messages')
+                assert False
+
+            id = extractids(req)[-1]
+
+            msgRead = f'{API}?action=readMessage&login={login}&domain={domain}&id={id}'
+            req = requests.get(msgRead).json()
+
+            html = req['htmlBody']
+            
+            if html == '':
+                return req['textBody']
+            else:
+                return  html
+
+        retry_if_is_error(
+            run, NETWORK_ERRORS + [AssertionError], 
+            # 1
+            RETRY_ATTEMPTS_HIGH
+            , WAIT_DURATION_VERY_SHORT)
+        data = run()
+        return data
+
+    def get_email_link_and_delete_mailbox(email):
+        TempMail.get_email_link(email)
+        TempMail.deleteMailbox(email)
 
 if __name__ == '__main__':
     print('Running')
